@@ -1,6 +1,6 @@
 use std::{future::Future, sync::Arc};
 
-use crate::{Endpoint, IntoResponse, Request};
+use crate::{Endpoint, IntoResponse, Request, Result};
 
 /// Endpoint for the [`around`](super::EndpointExt::around) method.
 pub struct Around<E, F> {
@@ -19,16 +19,16 @@ impl<E, F> Around<E, F> {
 }
 
 #[async_trait::async_trait]
-impl<E, F, Fut, R> Endpoint for Around<E, F>
+impl<E, F, Fut, T> Endpoint for Around<E, F>
 where
     E: Endpoint,
     F: Fn(Arc<E>, Request) -> Fut + Send + Sync + 'static,
-    Fut: Future<Output = R> + Send,
-    R: IntoResponse,
+    Fut: Future<Output = Result<T>> + Send,
+    T: IntoResponse,
 {
-    type Output = R;
+    type Output = T;
 
-    async fn call(&self, req: Request) -> Self::Output {
+    async fn call(&self, req: Request) -> Result<Self::Output> {
         (self.f)(self.inner.clone(), req).await
     }
 }
